

 /*+-------------------------------------------------------+
   |  SAS Global Forum 2009 Sample                         |
   |  SAS MAPPING: Tips and Tricks 4                       |          
   |                                                       |
   |    NAME: casinomap.sas                                |
   |   TITLE: Create a casino floor plan map.              |
   |          This program is used to generate a floor plan|
   |          map showing where the slot machines are on   |
   |          the floor and they can be colored according  |
   |          to the machine performance, use or payout.   |
   | PRODUCT: GRAPH                                        |
   | PAPER TITLE:                                          |
   |    Tips and Tricks IV:  More SAS/Graph Map Secrets    |
   |                                                       |
   |11Jan09                                                |
   +-------------------------------------------------------+*/

/*************************************************************
 * The floorplan consists of 3 pieces of data:
 *  1. An image of the floorplan drawing.  This image is 
 *     cropped on the outline borders. (casinofloor.gif)
 *  2. A map that is the same dimensions as the outside of 
 *     the floorplan drawing outline of the building. 
 *     (work.casino)
 *  3. A map showing the slot machine placement in the room.
 * All of this data was created outside of this program.
 *************************************************************/

filename odsout '.';
%let dev=png;   /*0utput device type*/
%let name=plan; /*html file name */


/*************************************************************
 * Get map of external walls of casino.
 *************************************************************
 * These coordinates are generated externally and would only 
 * need to be generated one time. 
 *************************************************************/
data work.casino;
   length id $20;
   input x y segment ID $;
   cards;
  0.00000000 320.03425125  1 Room 
  0.00000000   0.00000000  1 Room 
250.17123654   0.00000000  1 Room 
250.17123654 109.93150845  1 Room 
190.06849593 109.93150845  1 Room 
190.06849593 169.52055042  1 Room 
281.50685343 169.52055042  1 Room 
281.50685343 320.03425125  1 Room 
  0.00000000 320.03425125  1 Room 
;
run;


/*************************************************************
 * Create the map of the slot machine placement. 
 *************************************************************
 * The program below contains a macro that creates a series of 
 * rectangles in columns and rows, horizontal and 
 * vertical.  You must provide the starting location
 * placement.  The file also uses the macro and places the
 * slot machines in the casinos.  The coordinates were 
 * determined outside of this program.  The program will 
 * create the slot machine map: sasuser.slots
 * Normally, you would only need to do this one time.
 *************************************************************/
%include '.\makerectangle.sas';

/*************************************************************
 * Create a map data set that combines the room shape with
 * the slot machine placement.    
 *************************************************************/
data sasuser.casinofloor (keep=x y segment id htmlvar);
   set work.casino sasuser.slots;
run;

/*************************************************************
 * Get Slot machine usage data.  
 *************************************************************
 * We are just creating fake
 * example values here with a random number generator.    
 *************************************************************/
data usage;
   length ID $20;
   do i=1 to 100;
     ID="Slot "||trim(left(i));  /* Create a machine name/ID */
     Income=round(ranuni(0)*1000,1);
     output;
   end;
  run;
proc sort data=usage; by id; run;

/*************************************************************
 * Create tool tips for the usage data for each machine.
 * The slot machine placement map is merged in so that
 * the slot machine type can be displayed.  
 *************************************************************/
proc sort data=sasuser.slots; by id; run;
data usage2;
  merge usage sasuser.slots; by id;
  if (id ne 'Room') then /*skip the room*/
  htmlvar= 'title=' || quote(trim(left(id)) || '  (' || trim(left(type)) 
                  || ')  $' || left(trim(left(Income))));
  run;

/*************************************************************
 * Process the background 'blueprint' image.
 *************************************************************
 * In order to properly place the image, first determine
 * the size of the map data by finding the max/min
 * x and y values of the casino map. We know that our map 
 * starts at 0,0 and increases with positive numbers.  If your 
 * map is different then this logic will need changing.  
 *************************************************************/
data temp1;
   set work.casino;
   retain maxx 0 maxy 0;
   retain minx 9999999999 miny 999999999;
   if x <= minx and y <= miny then do;
      minx=x;
      miny=y;
   end;
   else if x >= maxx and y >= maxy then do;
      maxx=x;
      maxy=y;
   end;
run;

data temp1;  /* assign the max/min values to macro variables */
   set temp1;
   call symput("minx",minx);
   call symput("miny",miny);
   call symput("maxx",maxx);
   call symput("maxy",maxy);
run;


/*************************************************************
 * Create an annotate data set that draws the image of the 
 * casino floor plan under the casino map.
 *************************************************************/
data imageplan;
   length  function style color $ 8 html $ 1000 text $ 50;
   retain xsys '2' ysys '2' hsys '3' when 'b';
   retain color '' text '' html '';
   /*start with the minimum x,y values of the floor plan map data */
   function='move';x=&minx; y=&miny; output;  
   /*and draw the image to the max x,y values of the floor plan map data */
   function='image'; x=&maxx; y=&maxy; 
          imgpath='.\casinofloor.gif'; /* the floorplan image */
          style='fit';output;
run;

/*************************************************************
 * Draw the casino map.   
 *************************************************************/
goptions reset=all;
GOPTIONS DEVICE=&dev;
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._&dev..html";
  Title 'Casino 401K';
 
  /* Display the combined map with the annotate data set containing the image */
  proc gmap data=usage2 map=sasuser.casinofloor anno=imageplan ALL; 
      choro Income / nolegend html=htmlvar name="&name"; id id; 
  run; 
  quit;

ODS HTML CLOSE;
ODS LISTING;
