
 /*+-------------------------------------------------------+
   |  SAS Global Forum 2009 Sample                         |
   |  SAS MAPPING: Tips and Tricks 4                       |          
   |                                                       |
   |    NAME: makerectangle.sas                            |
   |   TITLE: Create rectangles by rows and columns        |
   |          given a starting coordinate.                 |
   | PRODUCT: GRAPH                                        |
   | PAPER TITLE:                                          |
   |    Tips and Tricks IV:  More SAS/Graph Map Secrets    |
   |                                                       |
   |11Jan09                                                |
   +-------------------------------------------------------+*/

/*************************************************************
 * Setup values for sizes of rectangles for macro.  You may 
 * need to adjust these for your own maps.
 *************************************************************/
%let width=14.05271785;  /*width of rectangle*/
%let depth=6.00706121;   /*depth of rectangle*/
%let wgap=0;             /*gap between rectangles width*/
%let dgap=0;             /*gap between rectangles height*/


/*************************************************************
 * Macro that does the work.  Arguments are:
 * xstart - starting x position.
 * ystart - starting y position.
 * across - number of columns of rectangles (width direction).
 * down   - number of rows of rectangles (depth direction).
 * direction - "V" or "H" - Vertical or horizontal.  Note 
 *             that rows, columns, width and height have the
 *             same meaning, but the whole thing is rotated
 *             in the map.
 * holes  - "Y" or "N".  If "Y" then create both the rectangle
 *          in the map and also create a 'hole' that the
 *          rectangle sits in.
 *************************************************************/
%macro makerectangle(xstart,ystart,across,down,direction,holes);
  startx =&xstart;
  starty= &ystart;
  segment=1;
  segsave=segment;
  savetype=type;  /*description of the type of rectangle*/
  
  /* loop through the rows */
  do j=1 to &down;
    /* loop through the columns */
    do i = 1 to &across; /*number of tables*/
	  slot=slot+1;	
        /* Vertical arrangement */	  
	  if &direction='V' then do;
            /*** Create holes and do them first ***/
		if &holes='Y' then do;
		  id='Room';  /*same as room map */
		  type='hole';
		  segcount=segcount+1;
		  segment=segcount;
	        x=.; y=.; output;  /* missing indicates a hole follows */
              /* all the way around the rectangle */
              x=startx; y=starty; output;
	        x=startx+&depth; y=starty; output;
	        x=startx+&depth; y=starty-&width; output;
	        x=startx;  y=starty-&width; output;
              x=startx; y=starty; output;
		end;
            /*** Now create the rectangle ***/
		segment=segsave;
		type=savetype;
            /* create an ID */
	      id='Slot '||trim(left(slot));
            /* Draw the rectangle */
            x=startx; y=starty; output;
	      x=startx+&depth; y=starty; output;
	      x=startx+&depth; y=starty-&width; output;
	      x=startx;  y=starty-&width; output;
            x=startx; y=starty; output;
		starty=starty-&width-&wgap; /*space them with the gap*/
		end;
        /* Horizontal "H" arrangement */
	  else do; 
            /*** Create holes and do them first ***/
		if &holes='Y' then do; 
		  segcount=segcount+1;
		  segment=segcount;
		  id="Room";  /* give it the ID of the outside map */
		  type='hole';
		  x=.; y=.; output;  /* missing starts a hole */
              /* coordinates for hole rectangle */
              x=startx; y=starty; output;
	        x=startx+&width; y=starty; output;
	        x=startx+&width; y=starty-&depth; output;
	        x=startx;  y=starty-&depth; output;
              x=startx; y=starty; output;
		end;
            /*** Now create the rectangle ***/
		segment=segsave;
		type=savetype;
	      id='Slot '||trim(left(slot));
            /* coordinates of the rectangle */
            x=startx; y=starty; output;
	      x=startx+&width; y=starty; output;
	      x=startx+&width; y=starty-&depth; output;
	      x=startx;  y=starty-&depth; output;
            x=startx; y=starty; output;
		startx=startx+&width+&wgap; /*leave a gap between*/
		end;
      end;
      /* increment for the next table */
	if &direction='V' then do;
	  starty=&ystart;
	  startx=&xstart+&depth+&dgap;
	end;
	else do;
	  startx=&xstart;
	  starty=&ystart-&depth-&dgap;
	end;
  end;
%mend;


/*************************************************************
 * Use the macro to create the coordinates for this example.
 *************************************************************/
Data sasuser.slots;
   retain slot 0;
   length type $ 25;
   length savetype $25;
   length id $20;
   retain type '';
   retain segsave 0;
   retain segcount 1;
   retain savetype '';

   /* create the $1 slots.  8 wide, 2 rows, horizontal with a hole */
   type='$1 Slots';
   %makerectangle(60.05069708,288.67869521,8,2,'H','Y');

   /* 3 groupings of machines.  7 wide, 2 rows, vertical with hole */
   type='Half dollar Slots';
   %makerectangle(17.61792934,218.85849839,7,2,'V','Y');
   %makerectangle(50.41374086,218.85849839,7,2,'V','Y');
   %makerectangle(82.45561457,218.85849839,7,2,'V','Y');

   /* 3 groupings of machines, 8 wide, 2 rows, horizontal with hole */
   type='Quarter Slots';
   %makerectangle(17.16713468,83.17910583,8,2,'H','Y');
   %makerectangle(17.16713468,49.44239467,6,2,'H','Y');
   %makerectangle(122.95645624,49.44239467,4,2,'H','Y');

   /* 2 single machines, one horizontal and one vertical with hole */
   type='$5 Slots';
   %makerectangle(28.71561855,290.26990707,1,1,'H','Y');
   %makerectangle(5.17813876,272.01033085,1,1,'V','Y');

   /* 4 single machines, horizontal with holes */
   type='$10 Slots';
   %makerectangle(110.51666567,217.89294372,1,1,'H','Y');
   %makerectangle(139.54284368,217.89294372,1,1,'H','Y');
   %makerectangle(110.51666567,179.06571859,1,1,'H','Y');
   %makerectangle(139.54284368,179.06571859,1,1,'H','Y');
run;

/*************************************************************
 * Add a tool tip to the slot machine.
 *************************************************************/
/*****
data sasuser.slots;
   set sasuser.slots;
   if type ne 'hole' then
   htmlvar= 'title=' || quote(trim(left(id)) || '  ' || trim(left(type)));
run;
****/
