 /*+-------------------------------------------------------+
   |  SAS Global Forum 2009 Sample                         |
   |  SAS MAPPING: Tips and Tricks 4                       |          
   |                                                       |
   |    NAME: iowabubble.sas                               |
   |   TITLE: Create the Iowa election Bubble map.         |
   | PRODUCT: GRAPH                                        |
   | PAPER TITLE:                                          |
   |    Tips and Tricks IV:  More SAS/Graph Map Secrets    |
   |                                                       |
   | NOTE:  You must first CIMPORT data before running     |
   |        this example.                                  |
   |                                                       |
   |02Feb09                                                |
   +-------------------------------------------------------+*/

/*************************************************************
 * Before running this program, make sure you:
 * 1. Specify the SGF libname where your response data is
 *    located.
 * 2. Run the following PROC CIMPORT at least once to import
 *    the response data.  You may need to modify the path to
 *    the .dpo file.
 ************************************************************/
libname sgf "<put-your-path-here>"; 

proc cimport infile="iowadata.dpo" data=sgf.iowadata; run;

/* The iowadata set contains a list of Iowa counties, winner,*/
/* and votes per candidate                                   */

filename odsout '.';
%let dev=png;    /*0utput device type*/
%let name=block; /*html file name */
goption reset=all dev=png xpixels=800 ypixels=600;

/* NY Times colors */
%let ccolor=cxc8cfe3;
%let ecolor=cxb8afce;
%let ocolor=cxc1cdab;
%let tcolor=cxe2b2a1;
%let mcolor=gray88;
%let pcolor=gray55;

/*************************************************************
 * Subset Iowa county data from the US county name data set 
 ************************************************************/
data nm;
  set maps.cntyname;
  where state=19; /*only Iowa*/
run;

/*************************************************************
 * Process the IOWADATA response data so that it can be used
 * later.  Add additional information.
 ************************************************************/
/* Create a temporary data set for merging.  To be sure 
 * things merge correctly, set the name for the county to 
 * all upper-case, since that is how they are stored in 
 * MAPS.CNTYNAME.  Retain the old mixed-case name for 
 * presentation purposes. 
 */
data t;
  set sgf.iowadata;
  cname=countynm;
  countynm=upcase(countynm);
run;

/* Sort so the merge happens correctly */
proc sort data=nm;
  by countynm; run;
proc sort data=t;
  by countynm; run;

/* Merge and add up all votes cast for sizing purposes */
data t;
  merge t nm(keep=countynm county);
  by countynm;
  total=clinton+obama+edwards;
run;

/* Calculate margin of victory.  First, you have to know which 
 * column is the winner, then subtract the max of the other two 
 */
data t;
  set t;
  if winner = 'C' then
     margin=clinton-max(edwards,obama);
  else if winner = 'E' then
     margin=edwards-max(clinton,obama);
  else if winner = 'O' then
     margin=obama-max(clinton,edwards);
  else margin=.;
run;

/************************************************************* 
 * Create an Iowa county map by projecting a subset from 
 * MAPS.COUNTIES 
 ************************************************************/
proc gproject data=maps.counties out=map;
  where state=19;
  id county;
run;

/*************************************************************
 * Create a user-defined format for each candidate 
 ************************************************************/
proc format;
  value $cand 'C'='Clinton' 'E'='Edwards' 'O'='Obama' 'T'='Tie';
run;

/*************************************************************
 * Draw a block-area map showing winner and margin of victory 
 ************************************************************/
GOPTIONS DEVICE=&dev ;
GOPTIONS CBACK='white';
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._&dev..html";
pattern1 c=&ccolor v=ms;
pattern2 c=&ecolor v=ms;
pattern3 c=&ocolor v=ms;
pattern4 c=&tcolor v=ms;
title '2008 Iowa Democratic Party Caucus';
title2 'Margin of Victory';
legend1 position=(top right) label=('Winner' font="Thorndale AMT/bold") 
   value=(font="Thorndale AMT/bold") across=1 mode=share frame;
proc gmap data=t map=map;
  id county;
  block margin / nolegend shape=cylinder name="&name";
  area winner / legend=legend1 ;
  format winner $cand.;
run;quit;
ODS HTML CLOSE;
ODS LISTING;

/*************************************************************
 * Further process the Response data to find margin of victory
 ************************************************************/
 /* Calulate biggest and smallest margin of victory */
proc summary data=t;
  var margin;
  output out=t2 min=mintotal max=maxtotal;run;

/* Save results as macro variables */
data _null_;
  set t2;
  call symput("min", mintotal);
  call symput("max", maxtotal);
run;

/*************************************************************
 * For location identification, find the largest cities in 
 * Iowa.  Cities that had a population over 50,000 in the 1990 
 * U.S. Census are subsetted from the USCITY data set and 
 * coordinates are converted to radians 
 *************************************************************/
data cities;
  set maps.uscity;
  where state=19 and pop_1990 > 50000;
  /* convert to Radians */
  x=atan(1)/45 * long;
  y=atan(1)/45 * lat;
  keep pop_1990 county city x y;
run;

/*************************************************************
 * Combine the points and map data sets, project the data and
 * split the data sets.
 *************************************************************/
/* Combine city points and county boundaries into single 
 * data set.
 */
data iamap;
  set maps.counties(where=(state=19)) cities;
run;

/* Project to make the map look nicer */
proc gproject data=iamap out=iamap;
  id county;
run;

/* Split the data back to cities and counties.  The city points
 * are going to go into an annotate data set, while the county
 * boundaries are the map data set 
 */
data sgf.iamap cities;
  set iamap;
  if city ne ' ' then output cities;
  else output sgf.iamap;
run;

/*************************************************************
 * Create 3 annotate data sets: Bubbles(pie), City labels, 
 * and legends.
 *************************************************************/
/* Bubbles(pie) Annotate data set.
 * Here's a little trick:  The annotate library already has a 
 * macro for labeling the centroid of polygons.  Use that to 
 * create the annotate data set and later change
 * the annotate function to 'pie' instead of 'label' 
 */
%annomac;
%maplabel (sgf.iamap, t, labelout, cname, county, 
      font='Thorndale AMT', color=cxA06000, size=2.5, hsys=3);

/* Update the annotate data set to have it do what we want.  
 * Change it to a pie
 */
data labelout;
  length html $160 function $8 style $20;
  retain rotate 360;
  merge labelout t;
  by county;
  when='B'; /* Make bubbles draw under the map */

  /* Change the function from 'label' to 'pie' */
  function='pie     ';
  style   ='psolid  ';
  line=1;
  width=1;
  rotate=360;

  /* Scale the size from 2 to 10 */
  size=2 + 8*(margin-&min)/(&max-&min);

  /* Size is missing when there was no clear winner.  We 
   * still want to see the pie, but make it as small as 
   * possible 
   */
  if size=. then size=1;

  /* Set the color and chart tip for Hillary Clinton */
  if winner = 'C' then do;
     color="&ccolor";
     html="title='" || trim(cname) || "County " || '0D'x || 
       "Hillary Clinton: " || left(put(margin, comma8.0)) || "'";
  end;

  /* Set the color and chart tip for John Edwards */
  else if winner = 'E' then do;
     color="&ecolor";
     html="title='" || trim(cname) || " County" || '0D'x || 
       "John Edwards: " || left(put(margin, comma8.0)) || "'";
  end;

  /* Set the color and chart tip for Barack Obama */
  else if winner = 'O' then do;
     color="&ocolor";
     html="title='" || trim(cname) || " County" || '0D'x || 
       "Barack Obama: " || left(put(obama, comma8.0)) || "'";
  end;

  /* Set the color and chart tip for a tie situation */
  else do;
     color="&tcolor";
     html="title='" || trim(cname) || " County" || '0D'x || 
       "Tied:  Total Votes=" || left(put(obama+clinton+edwards, comma8.0)) || "'";
  end;
  output;

  /* Draw an outline around the bubble */
  style   ='pempty';
  color   ="&pcolor";
  html    ='';
  output;
run;

/* Sort by descending size so that smaller pies show up on 
 * top of larger ones 
 */
proc sort data=labelout;
  by descending size;
run;

/*************************************************************
 * Cities Annotate data set.
 *
 * Find min/max size of cities 
 */
proc summary data=cities;
  var pop_1990;
  output out=t2 min=minpop max=maxpop;run;

/* Store size in macro variables */
data _null_;
  set t2;
  call symput("min", minpop);
  call symput("max", maxpop);
run;

/* Create an annotate data set for labeling the city locations */
data cities;
  length function $8 style $20;
  retain color 'gray40  ' hsys '3' xsys ysys '2' when 'A' style 
       'psolid' function 'pie' rotate 360 position '+' size .5  x y;
  set cities(rename=(city=text));

  /* Set the size from .3 to 1.1 based on population */
  size=.3 + .8*(pop_1990-&min)/(&max-&min);
  position='+';function='pie     ';style='psolid       ';
      hsys='3';ysys='2';output;
  function='cntl2txt';output;
  position='2';function='label   ';style='Thorndale AMT/bold';
      hsys='D';size=12;ysys='7';y=.75;output;
run;

/*************************************************************
 * Legend Annotate data set.
 *
 * Create a legend for the slice colors 
 */
data legend;
  length function $8 text style $20;
  retain hsys xsys ysys '3' function 'pie     ' when 'A' 
    position '+' line 1 size 1 x y rotate 360;
  x=95;y=90;style='psolid  ';text='U';color="&ccolor";output;
  style='pempty  ';color='black   ';output;
  y=86.5;style='psolid  ';color="&ecolor";output;
  style='pempty  ';color='black   ';output;
  y=83;style='psolid  ';color="&ocolor";output;
  style='pempty  ';color='black   ';output;
  y=79.5;style='psolid  ';color="&tcolor";output;
  style='pempty  ';color='black   ';output;
  x=94;y=90;function='label   ';style='Thorndale AMT/bold';
     text='Clinton';position='<';hsys='D';size=12;output;
  y=86.5;text='Edwards';output;
  y=83;text='Obama';output;
  y=79.5;text='Tie';output;

  /* Draw a frame around the legend. */
  x=96;y=92;color='black   ';function='move    ';
    size=.5;hsys='4';output;
  x=86;function='draw    ';output;
  y=78;function='draw    ';output;
  x=96;function='draw    ';output;
  y=92;function='draw    ';output;
run;

/*************************************************************
 * Put all three annotate datasets together.
 *************************************************************/
data labelout;
  set labelout legend cities;
run;

/*************************************************************
 * Draw the bubble map.
 ************************************************************/
%let name=bubble; /*html file name */
GOPTIONS reset=all;
GOPTIONS DEVICE=&dev;
GOPTIONS CBACK='white';
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._&dev..html";
title '2008 Iowa Democratic Party Caucus';
title2 'Margin of Victory';
pattern1 v=me c=&mcolor r=4;
proc gmap data=t map=sgf.iamap anno=labelout;
  id county;
  choro winner / nolegend name="&name";
run;
quit;

ODS HTML CLOSE;
ODS LISTING;
