 /*+-------------------------------------------------------+
   |  SAS Global Forum 2009 Sample                         |
   |  SAS MAPPING: Tips and Tricks 4                       |          
   |                                                       |
   |    NAME: iowapie.sas                                  |
   |   TITLE: Create the Iowa election pie map.            |
   | PRODUCT: GRAPH                                        |
   | PAPER TITLE:                                          |
   |    Tips and Tricks IV:  More SAS/Graph Map Secrets    |
   |                                                       |
   | NOTE:  You must first CIMPORT data before running     |
   |        this example.                                  |
   |                                                       |
   |02Feb09                                                |
   +-------------------------------------------------------+*/

/*************************************************************
 * Before running this program, make sure you:
 * 1. Specify the SGF libname where your response data is
 *    located.
 * 2. Run the following PROC CIMPORT at least once to import
 *    the response data. Note that the path to the .dpo file
 *    may need to be modified.
 ************************************************************/
libname sgf "<put-your-path-here>"; 

proc cimport infile="iowadata.dpo" data=sgf.iowadata; run;

/* The iowadata set contains a list of Iowa counties, winner,*/
/* and votes per candidate                                   */

filename odsout '.';
%let dev=png;    /*0utput device type*/
%let name=pie; /*html file name */
goption reset=all dev=png xpixels=800 ypixels=600;

/* NY Times colors:  These are the colors used by the NY Times 
 * website and work well for bubbles rather than pies.  When 
 * using pies, the higher-contrast colors are almost necessary 
 * to tell the difference between slices easily.
 *
 * Comment out the "high-contrast colors" section below to 
 * retain these colors 
 */
%let ccolor=cxc8cfe3;
%let ecolor=cxb8afce;
%let ocolor=cxc1cdab;
%let mcolor=gray88;
%let pcolor=gray55;

/* High contrast colors */
%let ccolor=cxd05b5b;
%let ecolor=cxd5c563;
%let ocolor=cx6f7eb3;
%let mcolor=gray55;
%let pcolor=gray33; 


/*************************************************************
 * Subset Iowa county data from the US county name data set 
 ************************************************************/
data nm;
  set maps.cntyname;
  where state=19; /*only Iowa*/
run;



/*************************************************************
 * Process the IOWADATA response data so that it can be used
 * later.  Add additional information.
 ************************************************************/
/* Create a temporary data set for merging.  To be sure 
 * things merge correctly, set the name for the county to 
 * all upper-case, since that is how they are stored in 
 * MAPS.CNTYNAME.  Retain the old mixed-case name for 
 * presentation purposes. 
 */
data t;
  set sgf.iowadata;
  cname=countynm;
  countynm=upcase(countynm);
run;

/* Sort so the merge happens correctly */
proc sort data=nm;
  by countynm; run;
proc sort data=t;
  by countynm; run;

/* Merge and add up all votes cast for sizing purposes */
data t;
  merge t nm(keep=countynm county);
  by countynm;
  total=clinton+obama+edwards;
run;

/*************************************************************
 * Further process the Response data to find total vote
 *************************************************************/
 /* Find the minimum/maximum total votes cast                */
proc summary data=t;
  var total;
  output out=t2 min=mintotal max=maxtotal;run;

/* Store results as macro variables */
data _null_;
  set t2;
  call symput("min", mintotal);
  call symput("max", maxtotal);
run;

/*************************************************************
 * For location identification, find the largest cities in 
 * Iowa.  Cities that had a population over 50,000 in the 1990 
 * U.S. Census are subsetted from the USCITY data set and 
 * coordinates are converted to radians 
 *************************************************************/
data cities;
  set maps.uscity;
  where state=19 and pop_1990 > 50000;
  /* convert to Radians */
  x=atan(1)/45 * long;
  y=atan(1)/45 * lat;
  keep pop_1990 county city x y;
run;



/*************************************************************
 * Combine the points and map data sets, project the data and
 * split the data sets.
 *************************************************************/
/* Combine city points and county boundaries into single 
 * data set.
 */
data iamap;
  set maps.counties(where=(state=19)) cities;
run;

/* Project to make the map look nicer */
proc gproject data=iamap out=iamap;
  id county;
run;

/* Split the data back to cities and counties.  The city points
 * are going to go into an annotate data set, while the county
 * boundaries are the map data set 
 */
data sgf.iamap cities;
  set iamap;
  if city ne ' ' then output cities;
  else output sgf.iamap;
run;

/*************************************************************
 * Create 3 annotate data sets: Pie, City labels, 
 * and legends.
 *************************************************************/
/* Pie Annotate data set.
 * Here's a little trick:  The annotate library already has a 
 * macro for labeling the centroid of polygons.  Use that to 
 * create the annotate data set and later change
 * the annotate function to 'pie' instead of 'label' 
 */
%annomac;
%maplabel (sgf.iamap, work.t, work.labelout, cname, county, 
           font='Thorndale AMT', color=cxA06000, size=2.5, hsys=3);


/* Update the annotate data set to have it do what we want.  
 * Change it to a pie
 */
data labelout;
  length html $160 function $8 style $20;
  retain rotate 360;
  merge labelout t;
  by county;

  when='B'; /* Make bubbles draw under the map */

  /* Change the function from 'label' to 'pie' */
  function='pie     ';
  style   ='psolid  ';
  line=1;
  width=1;

  /* Scale the size from a minimum of 1 to a maximum of 10 */
  size=1 + 9*(total-&min)/(&max-&min);

  /* Calculate the slice size for Hillary Clinton and set the 
   * appropriate color and chart tip 
   */
  rotate=360 * clinton / total;
  color="&ccolor";
  html="title='" || trim(cname) || "County " || '0D'x || 
    "Hillary Clinton: " || left(put(clinton, comma8.0)) || "'";
    output;

  /* Calculate the slice size for John Edwards and set the 
   * appropriate color and chart tip 
   */
  rotate=360 * edwards / total;
  color="&ecolor";
  html="title='" || trim(cname) || " County" || '0D'x || 
    "John Edwards: " || left(put(edwards, comma8.0)) || "'";
    output;

  /* Calculate the slice size for Barack Obama and set the 
   * appropriate color and chart tip 
   */
  rotate=360 * obama / total;
  color="&ocolor";
  html="title='" || trim(cname) || " County" || '0D'x || 
    "Barack Obama: " || left(put(obama, comma8.0)) || "'";
    output;

  /* Draw an outline around the entire pie */
  rotate=360;
  style   ='pempty';
  color   ="&pcolor";
  html    ='';
  output;
run;

/* Sort by descending size so that smaller pies show up on 
 * top of larger ones 
 */
proc sort data=labelout;
  by descending size;
run;

/*************************************************************
 * Cities Annotate data set.
 *
 * Find min/max size of cities 
 */
proc summary data=cities;
  var pop_1990;
  output out=t2 min=minpop max=maxpop;run;

/* Store in macro variables */
data _null_;
  set t2;
  call symput("min", minpop);
  call symput("max", maxpop);
run;

/* Create an annotate data set for labeling the city locations */
data cities;
  length function $8 style $20;
  retain color 'gray40  ' hsys '3' xsys ysys '2' when 'A' 
    style 'psolid' function 'pie' rotate 360 position '+' size .5 x y;
  set cities(rename=(city=text));

  /* Set the size from .3 to 1.1 based on population */
  size=.3 + .8*(pop_1990-&min)/(&max-&min);
  position='+';function='pie     ';style='psolid       ';
    hsys='3';ysys='2';output;
  function='cntl2txt';output;
  position='2';function='label   ';style='Thorndale AMT/bold';
    hsys='D';size=12;ysys='7';y=.75;output;
run;

/*************************************************************
 * Legend Annotate data set.
 *
 * Create a legend for the slice colors 
 */
data legend;
  length function $8 text style $20;
  retain hsys xsys ysys '3' function 'pie     ' when 'A' 
    position '+' line 1 size 1 x y rotate 360;
  x=95;y=90;style='psolid  ';text='U';color="&ccolor";output;
  style='pempty  ';color='black   ';output;
  y=86.5;style='psolid  ';color="&ecolor";output;
  style='pempty  ';color='black   ';output;
  y=83;style='psolid  ';color="&ocolor";output;
  style='pempty  ';color='black   ';output;
  x=94;y=90;function='label   ';style='Thorndale AMT/bold';
    text='Clinton';position='<';hsys='D';size=12;output;
  y=86.5;text='Edwards';output;
  y=83;text='Obama';output;

  /* Draw a frame around the legend. */
  x=96;y=92;color='black   ';function='move    ';size=.5;hsys='4';output;
  x=86;function='draw    ';output;
  y=80.5;function='draw    ';output;
  x=96;function='draw    ';output;
  y=92;function='draw    ';output;
run;

/*************************************************************
 * Put all three annotate datasets together.
 *************************************************************/
data labelout;
  set labelout legend cities;
run;

/*************************************************************
 * Draw the pie map with slices.
 ************************************************************/
GOPTIONS reset=all;
GOPTIONS DEVICE=&dev;
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._&dev..html";
goptions cback='white';
title '2008 Iowa Democratic Party Caucus';
title2 'Results by County';
pattern1 c=&mcolor v=me r=4;

proc gmap data=t map=sgf.iamap anno=labelout;
  id county;
  choro winner / nolegend name="&name";
run;
quit;

ODS HTML CLOSE;
ODS LISTING;
