 /*+-------------------------------------------------------+
   |  SAS Global Forum 2009 Sample                         |
   |  SAS MAPPING: Tips and Tricks 4                       |          
   |                                                       |
   |    NAME: sasu1.sas                                    |
   |   TITLE: Student Distribution                         |
   |          Show the distribution of enrolled and no     |
   |          longer enrolled students on a map.           |
   | PRODUCT: GRAPH                                        |
   | PAPER TITLE:                                          |
   |    Tips and Tricks IV:  More SAS/Graph Map Secrets    |
   |                                                       |
   | NOTE:  You must first CIMPORT data before running     |
   |        this example.                                  |
   |                                                       |
   |02Feb09                                                |
   +-------------------------------------------------------+*/

/*************************************************************
 * Before running this program, make sure you:
 * 1. Specify the SGF libname where your response data is
 *    located.
 * 2. Run the following PROC CIMPORT at least once to import
 *    the response data. Note that the path to the .dpo file
 *    may need to be modified.
 * 3. Download the reduce_pixel macro from the MapsOnLine
 *    web site and unzip it in your directory.  MapsOnLine is
 *    at http://support.sas.com/MapsOnLine.  The macro is
 *    under 'Resources' and then under 'Tools'.
 * 4. Specify the SGF Filename location where you downloaded
 *    the reduce_pixel program.
 ************************************************************/
libname sgf "<put-your-path-here>";
filename sgf "<put-your-path-to-programs-here>"; 

proc cimport infile="sasudata.dpo" data=sgf.sasudata; run;

/* Data set contains a student id, street address, */
/* and 5-digit zip code                            */

filename odsout '.';
%let dev=png;       /*0utput device type*/
%let name=sasu1;   /*html file name */
%let xpixels=800;
%let ypixels=600;

/*************************************************************
 * Continue to process the RESPONSE data.
 *************************************************************/
/* Proc GEOCODE converts zipcode to X/Y map values          */
proc geocode zip nocity attributevar=(city) 
     data=sgf.sasudata out=sgf.studzip;
run;

/* Convert from degrees to radians */
data sgf.studzip;
  set sgf.studzip;
  x=-x*atan(1)/45;
  y= y*atan(1)/45;
  drop _matched_;
run;

/*************************************************************
 *  Place all the student location points into an annotate 
 *  data set
 *************************************************************/
data anno_dots;
   length function color $ 8 style $20 position $ 1 text $ 60 html $1024;
   retain xsys ysys '2' hsys '3' when 'a' html '' position '';
   set sgf.studzip;
   anno_flag=1;

   /* All annotate functions are 360-degree pies.  They just look better. */
   function='pie';
   rotate=360;

   /* Make a special marker for the school location */
   if student_id='SCHOOL' then do;
     text='V';
     size=5;
     function='label';
     style='marker';
     color="blue";
     matric=0;
     output;
     style='markere';
     color='gray55';
   end;

   /* Student points */
   else do;
     size=.7;
     style='psolid';

     /* Still-matriculated use green (lime) color */
     if matric=1 then color='lime';

     /* No longer matriculated uses red color */
     else color='red';
     output;

     /* Draw an outline around each 'dot' */
     color="gray55"; style='pempty';
  end;
  output;
run;

/*************************************************************
 *  Process Map data.
 *************************************************************/
/* Create a map with only Texas (state=48)                   */
data txmap;
  length x y 8;
  set maps.counties;
  where state=48;
run;

/* Macro to reduce map data to 800x600 output */
%inc sgf(reduce_pixel);
%reduce(txmap, txmap, county, &xpixels, &ypixels );

/*************************************************************
 * Combine the points and map data sets, project the data and
 * split the data sets.
 *************************************************************/
/* Combine the data sets for projecting */
data txmap;
  set txmap anno_dots;
run;

/* Project */
proc gproject data=txmap out=txmap dupok;
  id county;
run;

/* Split the data into map and annotate data sets */
data txmap anno_dots;
  set txmap;
  if anno_flag > 0 then output anno_dots;
  else output txmap;
  drop anno_flag;
run;

/*************************************************************
 * Add an annotated legend.  This must be done after 
 * projecting since it uses screen coordinates, not map 
 * coordinates. 
 *************************************************************/
data t;
  retain xsys ysys hsys '3' position ' ' when 'a' size .7 rotate 360;
  length function color $ 8 style $ 20 text $60;

  /* Markers for the legend */
  color='lime    '; function='pie'; style='psolid';x=86.5;y=86.5;output;
  style='pempty'; color='gray55';output;
  color='red     '; function='pie'; style='psolid';y=84;output;
  style='pempty'; color='gray55';output;

  /* Draw a frame around the legend */
  function='move';hsys=' ';size=4;x=87.5;y=88;output;
  function='draw';x=69;color='black';output;
  function='draw';y=82;output;
  function='draw';x=87.5;output;
  function='draw';y=88;output;

  /* Text labels for the markers */
  color='black'; function='label'; style='Thorndale AMT/bold';
     position='<';text='Enrolled'; hsys='D';size=12;x=85;y=86.5;output;
  text='No longer enrolled';y=84;output;
run;

/* Add the legend to the annotations */
data anno_dots;
set anno_dots t;
run;

/*************************************************************
 * Draw the map with annotations
 ************************************************************/
GOPTIONS reset=all;
GOPTIONS DEVICE=&dev;
GOPTIONS xpixels=&xpixels ypixels=&ypixels;
GOPTIONS CBACK='white';
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._&dev..html";
title '2008 Freshman Class Retention';
title2 'Santa Anna State University';
pattern1 v=ms c=wheat;

proc gmap data=txmap map=txmap anno=anno_dots;
  id county;
  choro state / nolegend name="&name";
run;
quit;

ODS HTML CLOSE;
ODS LISTING;

