 /*+-------------------------------------------------------+
   |  SAS Global Forum 2009 Sample                         |
   |  SAS MAPPING: Tips and Tricks 4                       |          
   |                                                       |
   |    NAME: sasu2.sas                                    |
   |   TITLE: Student Distribution with Average distance   |
   |          circle on a map.                             |
   | PRODUCT: GRAPH                                        |
   | PAPER TITLE:                                          |
   |    Tips and Tricks IV:  More SAS/Graph Map Secrets    |
   |                                                       |
   | NOTE:  You must first CIMPORT data before running     |
   |        this example.                                  |
   |                                                       |
   |02Feb09                                                |
   +-------------------------------------------------------+*/

/*************************************************************
 * Before running this program, make sure you:
 * 1. Specify the SGF libname where your response data is
 *    located.
 * 2. Run the following PROC CIMPORT at least once to import
 *    the response data. Note that the path to the .dpo file
 *    may need to be modified.
 * 3. Download the reduce_pixel macro from the MapsOnLine
 *    web site and unzip it in your directory.  MapsOnLine is
 *    at http://support.sas.com/MapsOnLine.  The macro is
 *    under 'Resources' and then under 'Tools'.
 * 4. Specify the SGF Filename location where you downloaded
 *    the reduce_pixel program.
 ************************************************************/
libname sgf "<put-your-path-here>"; 
filename sgf "<put-your-path-to-programs-here>"; 


proc cimport infile="sasudata.dpo" data=sgf.sasudata; run;

/* Data set contains a student id, street address, */
/* and 5-digit zip code                            */

filename odsout '.';
%let dev=png;       /*0utput device type*/
%let name=sasu2;    /*html file name */
%let xpixels=800;
%let ypixels=600;

/*************************************************************
 * Continue to process the RESPONSE data.
 *************************************************************/
/* Proc GEOCODE converts zipcode to X/Y map values          */
proc geocode zip nocity attributevar=(city) data=sgf.sasudata 
     out=sgf.studzip;
run;

/* Convert from degrees to radians */
data sgf.studzip;
  set sgf.studzip;
  x=-x*atan(1)/45;
  y= y*atan(1)/45;
  drop _matched_;
run;

/* Calculate distance of each student from school */
data sgf.studzip;
  retain sx sy;
  set sgf.studzip;

  /* Save the school's X/Y (first obs) to use as the base point */
  if _n_ = 1 then do;
     sx=x;
     sy=y;
  end;

  /* GEODIST gets the distance from 2 sets of points.  From the
   * school to each student's home location.
   * 'RM' tells geodist that input units are Radians and 
   * output units are Miles 
   */
  else dist = geodist(y,x,sy,sx,'RM');
    drop sx sy;
run;

/* Calculate mean distance for all students */
proc summary data=sgf.studzip;
  var dist;
  output out=t mean=mndist;
run;

/* Set mean distance to local macro variable */
data _null_;
  set t;
  call symput('mndist', left(put(round(mndist), f3.0)));
run;

/*************************************************************
 *  Place all the student location points into an annotate 
 *  data set
 *************************************************************/
data anno_dots;
  length function color $ 8 style $20 position $ 1 text $ 60 html $1024;
  retain xsys ysys '2' hsys '3' when 'a';
  set sgf.studzip;
  anno_flag=1;

  /* All annotate functions are 360-degree pies.  They just look better. */
  function='pie';
  rotate=360;

  /* Make a special marker for the school location */
  if student_id='SCHOOL' then do;
    text='V';
    size=5;
    function='label';
    style='marker';
    color="blue";
    matric=0;
    output;
    style='markere';
    color='gray55';
  end;

  /* Student points */
  else do;
    size=.7;
    style='psolid';

    /* Still-matriculated use green (lime) color */
    if matric=1 then color='lime';

    /* No longer matriculated uses red color */
    else color='red';
    output;

    /* Draw an outline around each 'dot' */
    color="gray55"; style='pempty';
  end;
  output;
run;

/*************************************************************
 *  Create a distance circle in an annotate dataset.
 *************************************************************/
data circle;
  length text $25 color function $8 style $20;
  set sgf.studzip(obs=1 keep=x y);
  retain xsys ysys '2' hsys '3' anno_flag 2 size 4 when 'a' text ' ';
  
  /* Degree to radian constant */
  d2r=atan(1)/45;

  /* Miles per radian constant */
  r2m=3958.739565;

  /* Save off the X/Y for the school (first obs) to be used as the center point */
  xcen=x;
  ycen=y;

  /* Calculate the points to draw a circle */
  /* This must be done as a polygon rather than a pie function 
   * in order to maintain the correct distance after projecting.
   * It's quite possible that the circle is no longer a circle
   * but more of an ellipse once we project 
   */
  do degree=0 to 355 by 5;  

     /* Begin a new circle */
     if degree=0 then do;

        /* The circles will be solid yellow */
        function='poly';
        style='empty';
        line=1;
        end;

	 /* Continue drawing the circle */
     else do;

        /* Outline the circle with desired color */
        function='polycont';
        color="black";
        end;

     /* Calculate a point along the circle */
     y=arsin(cos(degree*d2r)*sin(&mndist/r2m)*cos(ycen)
          +cos(&mndist/r2m)*sin(ycen));
     x=xcen+arsin(sin(degree*d2r)*sin(&mndist/r2m)/cos(y));
     output;
  end;
  drop degree r2m d2r xcen ycen;
run;

/*************************************************************
 *  Process Map data.
 *************************************************************/
/* Create a map with only Texas (state=48)                   */
data txmap;
  length x y 8;
  set maps.counties;
  where state=48;
run;

/* Macro to reduce map data to 800x600 output */
%inc sgf(reduce_pixel);
%reduce(txmap, txmap, county, &xpixels, &ypixels );

/*************************************************************
 * Combine the points and map data sets, project the data and
 * split the data sets.
 *************************************************************/
/* Combine all the data sets for projecting */
data txmap;
  set txmap anno_dots circle;
run;

/* Project */
proc gproject data=txmap out=txmap dupok;
  id county;
run;

/* Split map data from annotate data. */
data txmap anno_dots;
  set txmap;
  if anno_flag > 0 then output anno_dots;
  else output txmap;
  drop anno_flag;
run;
 
/*************************************************************
 * Add an annotated legend.  This must be done after 
 * projecting since it uses screen coordinates, not map 
 * coordinates. 
 *************************************************************/
data t;
  retain xsys ysys hsys '3' position ' ' when 'a' size .7 rotate 360;
  length function color $ 8 style $ 20 text $60;

  /* Markers for the legend */
  color='lime    '; function='pie'; style='psolid';x=86.5;y=86.5;output;
  style='pempty'; color='gray55';output;
  color='red     '; function='pie'; style='psolid';y=84;output;
  style='pempty'; color='gray55';output;

  /* Draw a frame around the legend */
  function='move';hsys=' ';size=4;x=87.5;y=88;output;
  function='draw';x=65;color='black';output;
  function='draw';y=80;output;
  function='draw';x=87.5;output;
  function='draw';y=88;output;

  /* Text labels for the markers */
  color='black'; function='label'; style='Thorndale AMT/bold';
      position='<';text='Enrolled';hsys='D';size=12;x=83;y=86.5;output;
  text='No longer enrolled';y=84;output;
  text="Mean distance: &mndist miles"; y=81.5;x=87;output;
run;

/* Include the legend into the annotate data set */
data anno_dots;
  set anno_dots t;
run;

/*************************************************************
 * Draw the map with annotations
 ************************************************************/
GOPTIONS reset=all;
GOPTIONS DEVICE=&dev;
GOPTIONS xpixels=&xpixels ypixels=&ypixels;
GOPTIONS CBACK='white';
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._&dev..html";
title '2008 Freshman Class Retention';
title2 'Santa Anna State University';
pattern1 v=ms c=wheat;

proc gmap data=txmap map=txmap anno=anno_dots;
  id county;
  choro state / nolegend name="&name";
run;
quit;

ODS HTML CLOSE;
ODS LISTING;
